import type { Cornerstone, Image, ImageLoadObject } from '@dcm/cornerstone-core';

/** Loader configure options */
export interface LoaderConfigureOptions {
  beforeSend?: (
    xhr: XMLHttpRequest,
    imageId?: string,
    headers?: Record<string, any>,
    params?: Record<string, any>,
  ) => void;
}

export interface CornerstoneWebImageLoader {
  /**
   * Convert array buffer to image. Returns a promise that resolves to an Image object for the bytes in arrayBuffer
   *
   * @param {ArrayBuffer} arrayBuffer ArrayBuffer with bytes for a web image (e.g. JPEG, PNG, etc)
   * @returns {Promise<HTMLImageElement>} Promise that resolves to an Image object
   */
  arrayBufferToImage(arrayBuffer: ArrayBuffer): Promise<HTMLImageElement>;

  /**
   * Creates a cornerstone Image object for the specified Image and imageId
   *
   * @param {HTMLImageElement} image An Image
   * @param {string} imageId the imageId for this image
   * @returns {Image} Cornerstone Image Object
   */
  createImage(image: HTMLImageElement, imageId: string): Image;

  /**
   * Loads an image given a url to an image
   *
   * @param imageId Image ID
   * @returns {Promise<ImageLoadObject>} Image load object
   */
  loadImage(imageId: string): Promise<ImageLoadObject>;

  /**
   * Configure loader
   *
   * @param {LoaderConfigureOptions} options Loader configure options
   */
  configure(options: LoaderConfigureOptions);

  /** External libraries */
  external: {
    cornerstone: Cornerstone;
  };
}

declare const cornerstoneWebImageLoader: CornerstoneWebImageLoader;

declare module '@dcm/web-image-loader' {
  export = cornerstoneWebImageLoader;
}
